// This file was generated by ecm_setup_version(): DO NOT EDIT!

#ifndef PLASMA_VERSION_H
#define PLASMA_VERSION_H

#define PLASMA_VERSION_STRING "6.7.3"
#define PLASMA_VERSION_MAJOR 6
#define PLASMA_VERSION_MINOR 7
#define PLASMA_VERSION_PATCH 3
#define PLASMA_VERSION ((6<<16)|(7<<8)|(3))

#endif
